Package: partykit
Title: A Toolkit for Recursive Partytioning
Date: 2026-03-13
Version: 1.2-27
Authors@R: c(person(given = "Torsten", family = "Hothorn", role = c("aut", "cre"), email = "Torsten.Hothorn@R-project.org",
                    comment = c(ORCID = "0000-0001-8301-0471")),
             person(given = "Heidi", family = "Seibold", role = "ctb", email = "heidi@seibold.co",
                    comment = c(ORCID = "0000-0002-8960-9642")),
             person(given = "Achim", family = "Zeileis", role = "aut", email = "Achim.Zeileis@R-project.org",
                    comment = c(ORCID = "0000-0003-0918-3766")))
Description: A toolkit with infrastructure for representing, summarizing, and
  visualizing tree-structured regression and classification models. This
  unified infrastructure can be used for reading/coercing tree models from
  different sources ('rpart', 'RWeka', 'PMML') yielding objects that share
  functionality for print()/plot()/predict() methods. Furthermore, new and improved
  reimplementations of conditional inference trees (ctree()) and model-based
  recursive partitioning (mob()) from the 'party' package are provided based
  on the new infrastructure. A description of this package was published
  by Hothorn and Zeileis (2015) <https://jmlr.org/papers/v16/hothorn15a.html>.
Depends: R (>= 3.5.0), graphics, grid, libcoin (>= 1.0-0), mvtnorm
Imports: grDevices, stats, utils, survival, Formula (>= 1.2-1), inum
        (>= 1.0-0), rpart (>= 4.1-11)
Suggests: XML, rJava, sandwich, strucchange, vcd, TH.data (>= 1.0-3),
        mlbench, AER, coin (>= 1.1-0), party (>= 1.3-0), RWeka (>=
        0.4-19), datasets, parallel, psychotools (>= 0.3-0),
        psychotree, randomForest, knitr, bibtex
LazyData: yes
License: GPL-2 | GPL-3
URL: http://partykit.r-forge.r-project.org/partykit/
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-03-13 17:06:41 UTC; hothorn
Author: Torsten Hothorn [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8301-0471>),
  Heidi Seibold [ctb] (ORCID: <https://orcid.org/0000-0002-8960-9642>),
  Achim Zeileis [aut] (ORCID: <https://orcid.org/0000-0003-0918-3766>)
Maintainer: Torsten Hothorn <Torsten.Hothorn@R-project.org>
Repository: CRAN
Date/Publication: 2026-03-13 18:00:15 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 14:47:34 UTC; unix
Archs: partykit.so.dSYM
